<?php
/**
 * Set up Monit for certain services on a server.
 * 
 * @author Thom Pol
 * @copyright 2022 ISPConfig - Thom Pol
 * @example setup_monit.php
 * @example setup_monit.php --alert-email=user@example.com
 * @see
 * 
 */

function generatePassword($length = 8, $use_special = false) {
    // Verfügbare Zeichen für Passwort
    $available = "abcdefghjkmnopqrstuvwxyzABCDEFGHJKLMNPQRSTUVWXYZ123456789";
    if($use_special == true) {
$available .= '%_/-_&+.<>';
    }

    if($length < 1) {
$length = 8;
    }
    $passwd = "";

    $force_special = ($use_special == true ? mt_rand(1, $length) : 0);

    for($i=1;$i<=$length;$i++) {
// Passwort mit zufälligen Zeichen füllen bis Länge erreicht
if($i == $force_special) {
    $passwd .= substr($available, mt_rand(57, strlen($available) - 1), 1);
} else {
    $passwd .= substr($available, mt_rand(0, strlen($available) - 1), 1);
}
    }

    // kreiertes Passwort zurückgeben
    return $passwd;
}

print('Installing Monit' . "\n");
shell_exec('DEBIAN_FRONTEND="noninteractive" apt-get install -o Dpkg::Options::="--force-overwrite" -qq -y monit');
print('Configuring Monit.' . "\n");
$alertEmail = getopt(null, ["name:"]);
// Set up main config
$monitrc = file_get_contents('/etc/monit/monitrc');
str_replace('set daemon 120', 'set daemon 30', $monitrc);
file_put_contents('/etc/monit/monitrc', $monitrc);

// Set up config files for each service that shall be monitored
$installedServices = array(
	'filesystemSpace',
	'resources'
);

// Get all PHP versions that are installed
$existing_php_versions = array(
	'5.6',
	'7.0',
	'7.1',
	'7.2',
	'7.3',
	'7.4',
	'8.0',
	'8.1'
);
$installed_php_versions = array();
foreach ($existing_php_versions as $php_version) {
    $php_exists = shell_exec('which php' . $php_version);
    if ($php_exists != '') {
        $installed_php_versions[] = $php_version;
    }
}

foreach ($installed_php_versions as $curver) {
	$phpfpm = 'php' . $curver . '-fpm';
	array_push($servicesInstalled, $phpfpm);
}

// Check for all services if they exist and if so, add them to services array
$supportedServices = array(
    'sshd',
    'mariadb',
	'pure-ftpd-mysql',
    'apache2',
    'nginx',
    'named',
    'postfix',
    'dovecot',
    'rspamd'
);

foreach ($supportedServices as $service) {
    $service_exists = shell_exec('which ' . $service);
    if ($service_exists != '') {
        $installedServices[] = $service;
    }
}

$services = array_unique($installedServices);

// Set config directories
$confAvailableDir = '/etc/monit/conf-available/';
$confEnabledDir = '/etc/monit/conf-enabled/';
// Put config files in conf available directory
foreach ($services as $service) {
	if ($service == 'apache2') {
        $conf='check process apache with pidfile /var/run/apache2/apache2.pid
	start program = "/usr/bin/systemctl start apache2" with timeout 60 seconds
	stop program  = "/usr/bin/systemctl stop apache2"
	if failed port 80 for 2 cycles then restart
	if failed port 443 for 2 cycles then restart
	if 5 restarts within 5 cycles then timeout';
	}

	if ($service == 'nginx') {
        $conf='check process nginx with pidfile /var/run/nginx.pid
	start program = "/usr/bin/systemctl start nginx" with timeout 60 seconds
	stop program  = "/usr/bin/systemctl stop nginx"
	if failed port 80 for 2 cycles then restart
	if failed port 443 for 2 cycles then restart
	if 5 restarts within 5 cycles then timeout';
	}

	if ($service == 'mariadb') {
        $conf='check process mariadb with pidfile /var/run/mysqld/mysqld.pid
	start program = "/usr/bin/systemctl start mariadb"
	stop program = "/usr/bin/systemctl stop mariadb"
	if failed host 127.0.0.1 port 3306 then restart
	if 5 restarts within 5 cycles then timeout';
	}

	foreach($installed_php_versions as $curver) {
        $phpfpm = 'php' . $curver . '-fpm';
        if ($service == $phpfpm) {
            $conf='check process ' . $phpfpm . ' with pidfile /var/run/php/' . $phpfpm . '.pid
	start program = "/usr/bin/systemctl start ' . $phpfpm .'" with timeout 60 seconds
	stop program  = "/usr/bin/systemctl stop ' . $phpfpm . '"
	if failed unixsocket /var/run/php/' . $phpfpm . '.sock then restart
	if 5 restarts within 5 cycles then timeout';
        }
	}

	if ($service == 'pure-ftpd-mysql') {
        $conf='check process pure-ftpd-mysql with pidfile /var/run/pure-ftpd/pure-ftpd.pid
	start program = "/usr/bin/systemctl start pure-ftpd-mysql"
	stop program  = "/usr/bin/systemctl stop pure-ftpd-mysql"
	if failed port 21 protocol ftp then restart
	if 5 restarts within 5 cycles then timeout';
	}


	if ($service == 'sshd') {
        $conf='check process sshd with pidfile /var/run/sshd.pid
	start program  "/usr/bin/systemctl start sshd"
	stop program  "/usr/bin/systemctl stop sshd"
	if failed port 22 protocol ssh then restart
	if 5 restarts within 5 cycles then timeout';
	}

	if ($service == 'named') {
        $conf='check process named with pidfile /var/run/named/named.pid
	start program = "/usr/bin/systemctl start named"
	stop program  = "/usr/bin/systemctl stop named"
	if failed port 53 use type udp protocol dns then restart
	if 5 restarts within 5 cycles then timeout';
	}

	if ($service == 'postfix') {
        $conf='check process postfix with pidfile /var/spool/postfix/pid/master.pid
	group mail
	start program = "/usr/bin/systemctl start postfix"
	stop  program = "/usr/bin/systemctl stop postfix"
	if failed port 25 protocol smtp then restart
	if 5 restarts within 5 cycles then timeout';
	}

	if ($service == 'dovecot') {
        $conf='check process dovecot with pidfile /var/run/dovecot/master.pid
	group mail
	start program = "/usr/bin/systemctl start dovecot"
	stop program = "/usr/bin/systemctl stop dovecot"
	#if failed host mail.yourdomain.tld port 993 type tcpssl sslauto protocol
	if failed port 993 then restart
	if 5 restarts within 5 cycles then timeout';
	}

	if ($service == 'rspamd') {
        $conf='check process rspamd
	matching \'rspamd: main process\'
	start program = "/usr/bin/systemctl start rspamd"
	stop program = "/usr/bin/systemctl start rspamd"
	
	if cpu is greater than 40% for 2 cycles then alert
	if cpu > 60% for 5 cycles then alert
	if memory > 80% for 4 cycles then alert
	if totalmem > 1024 MB for 5 cycles then alert';
	}

	if ($service == 'filesystemSpace') {
        $conf='check filesystem rootfs with path /
	if space usage > 90% then alert';
	}

	if ($service == 'resources') {
        $conf='if cpu usage > 95% for 10 cycles then alert
if memory usage > 75% for 5 cycles then alert';
	}

    if ($conf != '') {
        file_put_contents($confAvailableDir . $service, $conf);
    }
}

// Set up alert config
$monitEmail = getopt(null, ["alert-email:"]);
$service = 'alerts';
if (!empty($monitEmail)) {
	if (filter_var($monitEmail, FILTER_VALIDATE_EMAIL)) {
        $conf = 'set mailserver localhost
#set mailserver smtp.example.com port 587
	#username "user@example.com" password "welcome"

set alert ' . $monitEmail;
	} else {
        print('E-mail address for Monit alerts is invalid. Set up alerts manually in /etc/monit/conf-available/alerts');
	};
} else {
    $conf = '#set mailserver localhost
#set mailserver smtp.example.com port 587
	#username "user@example.com" password "welcome"
	
#set alert admin@example.com';
}

file_put_contents($confAvailableDir . $service, $conf);

$services[] = $service;

// Configure main config with UI and admin + pass
$service = "webui";
$monitpw = '';
$monitpw = generatePassword(12);
$conf='set httpd port 2812 and
	#SSL ENABLE
	#PEMFILE /usr/local/ispconfig/interface/ssl/ispserver.pem
	allow admin:' . $monitpw;
	
file_put_contents($confAvailableDir . $service, $conf);

$services[] = $service;

foreach ($services as $confFile) {
	shell_exec('ln -s ' . $confAvailableDir . $confFile . ' ' . $confEnabledDir);
}

// Restart Monit to apply new configuration
shell_exec('systemctl restart monit');

print('Your Monit admin password is ' . $monitpw);
?>